package Enigma;

public class Enigma {
	
	// 5 Walzen (https://de.wikipedia.org/wiki/Enigma_(Maschine)#Aufbau)
	private final String[] W = {
			"EKMFLGDQVZNTOWYHXUSPAIBRCJ",
			"AJDKSIRUXBLHWTMCQGZNPYFVOE",
			"BDFHJLCPRTXVZNYEIWGAKMUSQO",
			"ESOVPZJAYQUIRHXLNFTGKDCMWB",
			"VZBRGITYUPSDNHLXAWMJQOFECK"
	};
	
	// 3 Umkehrwalzen (https://de.wikipedia.org/wiki/Enigma_(Maschine)#Aufbau)
	private final String[] UKW = {
			"EJMZALYXVBWFCRQUONTSPIKHGD",
			"YRUHQSLDPXNGOKMIEBFZCWVJAT",
			"FVPJIAOYEDRZXWGCTKUQSBNMHL"
	};
	
	private Walze[] w;
	private Walze ukw;
	private Steckbrett s;
	
	/**
	 * @param config Leerzeichen getrennter String:
	 * 	1. Anzahl Walzen
	 * 	2. Walzenkonfiguration jeweils:
	 * 		2.1. Walzennummer (0-5)
	 * 		2.2. Walzenring (0-25)
	 * 		2.3. Walzenstellung (0-25)
	 * 	3. Umkehrwalze (0-2)
	 * 	4. Steckbrett, leerzeichengetrennt Buchstabenpaare
	 * 
	 * 	Achtung: noch keine Fehlerbehandlung! Steckbrett -> Buchstaben dürfen nicht doppelt vorkommen!
	 */
	public Enigma(String config) {
		String[] c = config.split(" ");
		
		int walzenanzahl = Integer.parseInt(c[0]);
		
		this.w = new Walze[walzenanzahl];
		
		int i = 1;
		for (int j = 0; j < walzenanzahl; j++) {
			int walze = Integer.parseInt(c[i++]);
			int ring = Integer.parseInt(c[i++]);
			int stellung = Integer.parseInt(c[i++]);
			
			this.w[j] = new Walze(this.W[walze], ring, stellung);
		}
		
		int ukw = Integer.parseInt(c[i++]);
		this.ukw = new Walze(this.UKW[ukw], 0, 0);
		
		String s = "";
		for (int j = i; j < c.length; j++) {
			if (j != i) s += " ";
			s += c[j];
		}
		
		this.s = new Steckbrett(s);
	}
	
	/**
	 * 
	 * @param s
	 * @return
	 */
	public String crypt(String s) {
		return this.crypt(s, true);
	}
	
	/**
	 * ver- und entschlüsselt einen Text
	 * @param s String der verschlüsselt werden soll 
	 * @param hidespecial sollen Zeichen, die keine Buchstaben sind ausgeblendet werden?
	 * @return ver- bzw. entschlüsselter Text
	 */
	public String crypt(String s, boolean hidespecial) {
		String ret = "";
		
		for (int i = 0; i < s.length(); i++) {
			// einzelner Buchstabe aus Text
			char c = s.toUpperCase().charAt(i);
			
			// wenn kein Buchstabe, dann ggf. überspringen oder unverschlüsselt übernehmen
			if (c < 'A' || c > 'Z') {
				if (!hidespecial) ret += c;
				continue;
			}
			
			// Steckbrett durchlaufen
			c = this.s.in(c);
			
			// Walzen in Hinrichtung durchlaufen
			for (int w = 0; w < this.w.length; w++) {
				c = this.w[w].in(c);
			}
			
			// Umkehrwalze durchlaufen
			c = this.ukw.in(c);
			
			// Walzen in Rückrichtung durchlaufen
			for (int w = this.w.length - 1; w >= 0; w--) {
				c = this.w[w].out(c);
			}
			
			// Steckbrett nochmals durchlaufen
			c = this.s.in(c);
			
			// verschlüsselter Buchstabe an Ergebnis anhängen
			ret += c;
			
			// Walzen weiter drehen
			for (int w = 0; w < this.w.length; w++) {
				if (!this.w[w].next()) break;
			}
			
			// zur besseren Übersicht Leerzeichen einfügen 
			if (i % 5 == 4 && hidespecial) ret += ' ';
		}
		
		return ret;
	}

}
